@@ -1,8 +1,5 @@
-use crate::{
-    bpf_loader, bpf_loader_deprecated, config, decode_error::DecodeError, feature, hash::hashv,
-    secp256k1_program, stake, system_program, sysvar, vote,
-};
-
+#![allow(clippy::integer_arithmetic)]
+use crate::{decode_error::DecodeError, hash::hashv};
 use borsh::{BorshDeserialize, BorshSchema, BorshSerialize};
 use num_derive::{FromPrimitive, ToPrimitive};
 use std::{
@@ -21,8 +18,6 @@ pub const MAX_SEEDS: usize = 16;
 /// Maximum string length of a base58 encoded pubkey
 const MAX_BASE58_LEN: usize = 44;
 
-const PDA_MARKER: &[u8; 21] = b"ProgramDerivedAddress";
-
 #[derive(Error, Debug, Serialize, Clone, PartialEq, FromPrimitive, ToPrimitive)]
 pub enum PubkeyError {
     /// Length of the seed is too long for address generation
@@ -30,8 +25,6 @@ pub enum PubkeyError {
     MaxSeedLengthExceeded,
     #[error("Provided seeds do not result in a valid address")]
     InvalidSeeds,
-    #[error("Provided owner is not allowed")]
-    IllegalOwner,
 }
 impl<T> DecodeError<T> for PubkeyError {
     fn type_of() -> &'static str {
@@ -75,11 +68,13 @@ pub enum ParsePubkeyError {
     WrongSize,
     #[error("Invalid Base58 string")]
     Invalid,
+    #[error("Infallible")]
+    Infallible,
 }
 
 impl From<Infallible> for ParsePubkeyError {
     fn from(_: Infallible) -> Self {
-        unreachable!("Infallible unihnabited");
+        Self::Infallible
     }
 }
 
@@ -164,16 +159,8 @@ impl Pubkey {
             return Err(PubkeyError::MaxSeedLengthExceeded);
         }
 
-        let owner = owner.as_ref();
-        if owner.len() >= PDA_MARKER.len() {
-            let slice = &owner[owner.len() - PDA_MARKER.len()..];
-            if slice == PDA_MARKER {
-                return Err(PubkeyError::IllegalOwner);
-            }
-        }
-
         Ok(Pubkey::new(
-            hashv(&[base.as_ref(), seed.as_ref(), owner]).as_ref(),
+            hashv(&[base.as_ref(), seed.as_ref(), owner.as_ref()]).as_ref(),
         ))
     }
 
@@ -213,10 +200,6 @@ impl Pubkey {
             }
         }
 
-        if program_id.is_native_program_id() {
-            return Err(PubkeyError::IllegalOwner);
-        }
-
         // Perform the calculation inline, calling this from within a program is
         // not supported
         #[cfg(not(target_arch = "bpf"))]
@@ -225,7 +208,7 @@ impl Pubkey {
             for seed in seeds.iter() {
                 hasher.hash(seed);
             }
-            hasher.hashv(&[program_id.as_ref(), PDA_MARKER]);
+            hasher.hashv(&[program_id.as_ref(), "ProgramDerivedAddress".as_ref()]);
             let hash = hasher.result();
 
             if bytes_are_curve_point(hash) {
@@ -306,10 +289,9 @@ impl Pubkey {
                 {
                     let mut seeds_with_bump = seeds.to_vec();
                     seeds_with_bump.push(&bump_seed);
-                    match Self::create_program_address(&seeds_with_bump, program_id) {
-                        Ok(address) => return Some((address, bump_seed[0])),
-                        Err(PubkeyError::InvalidSeeds) => (),
-                        _ => break,
+                    if let Ok(address) = Self::create_program_address(&seeds_with_bump, program_id)
+                    {
+                        return Some((address, bump_seed[0]));
                     }
                 }
                 bump_seed[0] -= 1;
@@ -367,22 +349,6 @@ impl Pubkey {
         #[cfg(not(target_arch = "bpf"))]
         crate::program_stubs::sol_log(&self.to_string());
     }
-
-    pub fn is_native_program_id(&self) -> bool {
-        let all_program_ids = [
-            bpf_loader::id(),
-            bpf_loader_deprecated::id(),
-            feature::id(),
-            config::program::id(),
-            stake::program::id(),
-            stake::config::id(),
-            vote::program::id(),
-            secp256k1_program::id(),
-            system_program::id(),
-            sysvar::id(),
-        ];
-        all_program_ids.contains(self)
-    }
 }
 
 impl AsRef<[u8]> for Pubkey {
@@ -391,6 +357,12 @@ impl AsRef<[u8]> for Pubkey {
     }
 }
 
+impl AsMut<[u8]> for Pubkey {
+    fn as_mut(&mut self) -> &mut [u8] {
+        &mut self.0[..]
+    }
+}
+
 impl fmt::Debug for Pubkey {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "{}", bs58::encode(self.0).into_string())
@@ -513,7 +485,7 @@ mod tests {
     fn test_create_program_address() {
         let exceeded_seed = &[127; MAX_SEED_LEN + 1];
         let max_seed = &[0; MAX_SEED_LEN];
-        let program_id = Pubkey::from_str("BPFLoaderUpgradeab1e11111111111111111111111").unwrap();
+        let program_id = Pubkey::from_str("BPFLoader1111111111111111111111111111111111").unwrap();
         let public_key = Pubkey::from_str("SeedPubey1111111111111111111111111111111111").unwrap();
 
         assert_eq!(
@@ -527,25 +499,25 @@ mod tests {
         assert!(Pubkey::create_program_address(&[max_seed], &program_id).is_ok());
         assert_eq!(
             Pubkey::create_program_address(&[b"", &[1]], &program_id),
-            Ok("BwqrghZA2htAcqq8dzP1WDAhTXYTYWj7CHxF5j7TDBAe"
+            Ok("3gF2KMe9KiC6FNVBmfg9i267aMPvK37FewCip4eGBFcT"
                 .parse()
                 .unwrap())
         );
         assert_eq!(
-            Pubkey::create_program_address(&["☉".as_ref(), &[0]], &program_id),
-            Ok("13yWmRpaTR4r5nAktwLqMpRNr28tnVUZw26rTvPSSB19"
+            Pubkey::create_program_address(&["☉".as_ref()], &program_id),
+            Ok("7ytmC1nT1xY4RfxCV2ZgyA7UakC93do5ZdyhdF3EtPj7"
                 .parse()
                 .unwrap())
         );
         assert_eq!(
             Pubkey::create_program_address(&[b"Talking", b"Squirrels"], &program_id),
-            Ok("2fnQrngrQT4SeLcdToJAD96phoEjNL2man2kfRLCASVk"
+            Ok("HwRVBufQ4haG5XSgpspwKtNd3PC9GM9m1196uJW36vds"
                 .parse()
                 .unwrap())
         );
         assert_eq!(
-            Pubkey::create_program_address(&[public_key.as_ref(), &[1]], &program_id),
-            Ok("976ymqVnfE32QFe6NfGDctSvVa36LWnvYxhU6G2232YL"
+            Pubkey::create_program_address(&[public_key.as_ref()], &program_id),
+            Ok("GUs5qLUfsEHkcMB9T38vjr18ypEhRuNWiePW2LoK4E3K"
                 .parse()
                 .unwrap())
         );
@@ -592,40 +564,4 @@ mod tests {
             );
         }
     }
-
-    #[test]
-    fn test_is_native_program_id() {
-        assert!(bpf_loader::id().is_native_program_id());
-        assert!(bpf_loader_deprecated::id().is_native_program_id());
-        assert!(config::program::id().is_native_program_id());
-        assert!(feature::id().is_native_program_id());
-        assert!(secp256k1_program::id().is_native_program_id());
-        assert!(stake::program::id().is_native_program_id());
-        assert!(stake::config::id().is_native_program_id());
-        assert!(system_program::id().is_native_program_id());
-        assert!(sysvar::id().is_native_program_id());
-        assert!(vote::program::id().is_native_program_id());
-    }
-
-    fn pubkey_from_seed_by_marker(marker: &[u8]) -> Result<Pubkey, PubkeyError> {
-        let key = Pubkey::new_unique();
-        let owner = Pubkey::default();
-
-        let mut to_fake = owner.to_bytes().to_vec();
-        to_fake.extend_from_slice(marker);
-
-        let seed = &String::from_utf8(to_fake[..to_fake.len() - 32].to_vec()).expect("not utf8");
-        let base = &Pubkey::try_from_slice(&to_fake[to_fake.len() - 32..]).unwrap();
-
-        Pubkey::create_with_seed(&key, seed, base)
-    }
-
-    #[test]
-    fn test_create_with_seed_rejects_illegal_owner() {
-        assert_eq!(
-            pubkey_from_seed_by_marker(PDA_MARKER),
-            Err(PubkeyError::IllegalOwner)
-        );
-        assert!(pubkey_from_seed_by_marker(&PDA_MARKER[1..]).is_ok());
-    }
 }
